@@ -1,6 +1,6 @@
 /***************************************************************************
  * full_test.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -20,15 +20,14 @@
 #include <time.h>
 #include <locale.h>
 #include <time.h>
-#if !defined(_MSC_VER) && !defined(__BORLANDC__)
 #include <getopt.h>
-#endif
 
 #include "mgl2/mgl.h"
 #include "mgl2/font.h"
 #include "mgl2/eval.h"
 //-----------------------------------------------------------------------------
-void mgl_create_cpp_font(HMGL gr, const wchar_t *how);
+void MGL_EXPORT mgl_create_cpp_font(HMGL gr, const wchar_t *how);
+long MGL_EXPORT mgl_check_tex_table();
 //-----------------------------------------------------------------------------
 struct mglSample	/// Structure for list of samples
 {
@@ -75,7 +74,6 @@ void test(mglGraph *gr)
 	return;
 }
 //-----------------------------------------------------------------------------
-#if !defined(_MSC_VER) && !defined(__BORLANDC__)
 static struct option longopts[] =
 {
 	{ "big",	no_argument,	&big,		1 },
@@ -156,7 +154,6 @@ void usage()
 		"--fexport	- test most of output formats\n"
 	);
 }
-#endif
 //-----------------------------------------------------------------------------
 void save(mglGraph *gr,const char *name,const char *suf="")
 {
@@ -389,15 +386,6 @@ void smgl_fexport(mglGraph *gr)	// test file export
 	gr->ImportMGLD("fexport.mgld");
 }
 //-----------------------------------------------------------------------------
-extern mglTeXsymb mgl_tex_symb[];
-extern long mgl_tex_num;
-int MGL_LOCAL_PURE mgl_tex_symb_cmp(const void *a, const void *b)
-{
-	const mglTeXsymb *aa = (const mglTeXsymb *)a;
-	const mglTeXsymb *bb = (const mglTeXsymb *)b;
-	return wcscmp(aa->tex, bb->tex);
-}
-//-----------------------------------------------------------------------------
 int main(int argc,char **argv)
 {
 	mgl_suppress_warn(true);
@@ -498,14 +486,7 @@ int main(int argc,char **argv)
 	{	smgl_fexport(gr);	delete gr;	return 0;	}
 	else if(dotest==5)
 	{
-		long i=0;	while(mgl_tex_symb[i].tex[0])	i++;
-		if(mgl_tex_num!=i)	printf("real=%lu, set=%ld\n",i,mgl_tex_num);
-		for(long i=0;mgl_tex_symb[i].tex[0];i++)
-		{
-				mglTeXsymb tst, *rts;	tst.tex = mgl_tex_symb[i].tex;
-				rts = (mglTeXsymb *) bsearch(&tst, mgl_tex_symb, mgl_tex_num, sizeof(mglTeXsymb), mgl_tex_symb_cmp);
-				if(!rts)	printf("Bad '%ls' at %lu\n",mgl_tex_symb[i].tex,i);
-		}
+		mgl_check_tex_table();
 		delete gr;	return 0;
 	}
 
