@@ -1,6 +1,6 @@
 /***************************************************************************
  * data.h is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -23,8 +23,6 @@
 #include "mgl2/data_cf.h"
 #include "mgl2/pde.h"
 //-----------------------------------------------------------------------------
-#include <vector>
-#include <string>
 #include <stdarg.h>
 //-----------------------------------------------------------------------------
 mreal MGL_EXPORT mglLinear(const mreal *a, long nx, long ny, long nz, mreal x, mreal y, mreal z);
@@ -135,9 +133,9 @@ using mglDataA::Momentum;
 	{	if(d.size()>0)	{	Create(d.size());	for(long i=0;i<nx;i++)	a[i] = d[i];	}
 		else	Create(1);	}
 	inline void Set(const std::vector<float> &d)
-	{	if(d.size()>0)	Set(&(a[0]),d.size());	else	Create(1);	}
+	{	if(d.size()>0)	Set(&(d[0]),d.size());	else	Create(1);	}
 	inline void Set(const std::vector<double> &d)
-	{	if(d.size()>0)	Set(&(a[0]),d.size());	else	Create(1);	}
+	{	if(d.size()>0)	Set(&(d[0]),d.size());	else	Create(1);	}
 	/// Allocate memory and set data from variable argument list of double values
 	inline void SetList(long n, ...)
 	{
@@ -364,6 +362,9 @@ using mglDataA::Momentum;
 	/// Normalize the data to range [v1,v2] slice by slice
 	inline void NormSl(mreal v1=0,mreal v2=1,char dir='z',bool keep_en=true,bool sym=false)
 	{	mgl_data_norm_slice(this,v1,v2,dir,keep_en,sym);	}
+	/// Limit the data to be inside [-v,v], keeping the original sign
+	inline void Limit(mreal v)
+	{	mgl_data_limit(this, v);	}
 
 	/// Apply Hankel transform
 	inline void Hankel(const char *dir)	{	mgl_data_hankel(this,dir);	}
@@ -392,34 +393,6 @@ using mglDataA::Momentum;
 	inline mglData Solve(mreal val, char dir, const mglData &i0, bool norm=true) const
 	{	return mglData(true,mgl_data_solve(this, val, dir, &i0, norm));	}
 
-	/// Interpolate by cubic spline the data to given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
-	inline mreal Spline(mreal x,mreal y=0,mreal z=0) const
-	{	return value(x,y,z);	}
-	/// Interpolate by cubic spline the data to given point x,\a y,\a z which normalized in range [0, 1]
-	inline mreal Spline1(mreal x,mreal y=0,mreal z=0) const
-	{	return value(x*(nx-1),y*(ny-1),z*(nz-1));	}
-	/// Interpolate by linear function the data to given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
-	inline mreal Linear(mreal x,mreal y=0,mreal z=0)	const
-	{	return mgl_data_linear_ext(this,x,y,z,0,0,0);	}
-	/// Interpolate by line the data to given point x,\a y,\a z which normalized in range [0, 1]
-	inline mreal Linear1(mreal x,mreal y=0,mreal z=0) const
-	{	return mgl_data_linear_ext(this,x*(nx-1),y*(ny-1),z*(nz-1),0,0,0);	}
-
-	/// Interpolate by cubic spline the data and return its derivatives at given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
-	inline mreal Spline(mglPoint &dif, mreal x,mreal y=0,mreal z=0) const
-	{	return valueD(x,y,z, &(dif.x),&(dif.y), &(dif.z));	}
-	/// Interpolate by cubic spline the data and return its derivatives at given point x,\a y,\a z which normalized in range [0, 1]
-	inline mreal Spline1(mglPoint &dif, mreal x,mreal y=0,mreal z=0) const
-	{	mreal res=valueD(x*(nx-1),y*(ny-1),z*(nz-1), &(dif.x),&(dif.y), &(dif.z));
-		dif.x*=nx>1?nx-1:1;	dif.y*=ny>1?ny-1:1;	dif.z*=nz>1?nz-1:1;	return res;	}
-	/// Interpolate by linear function the data and return its derivatives at given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
-	inline mreal Linear(mglPoint &dif, mreal x,mreal y=0,mreal z=0)	const
-	{	return mgl_data_linear_ext(this,x,y,z, &(dif.x),&(dif.y), &(dif.z));	}
-	/// Interpolate by line the data and return its derivatives at given point x,\a y,\a z which normalized in range [0, 1]
-	inline mreal Linear1(mglPoint &dif, mreal x,mreal y=0,mreal z=0) const
-	{	mreal res=mgl_data_linear_ext(this,x*(nx-1),y*(ny-1),z*(nz-1), &(dif.x),&(dif.y), &(dif.z));
-		dif.x*=nx>1?nx-1:1;	dif.y*=ny>1?ny-1:1;	dif.z*=nz>1?nz-1:1;	return res;	}
-
 	/// Copy data from other mglData variable
 	inline const mglDataA &operator=(const mglDataA &d)
 	{	if(this!=&d)	mgl_data_set(this,&d);	return d;	}
@@ -542,6 +515,16 @@ inline mglData mglRay(const char *ham, mglPoint r0, mglPoint p0, mreal dt=0.1, m
 /// Saves result of ODE solving (|u|^2) for "Hamiltonian" ham with initial conditions ini
 inline mglData mglODE(const char *df, const char *var, const mglDataA &ini, mreal dt=0.1, mreal tmax=10)
 {	return mglData(true, mgl_ode_solve_str(df,var, &ini, dt, tmax));	}
+//-----------------------------------------------------------------------------
+/// Get array as solution of tridiagonal system of equations a[i]*x[i-1]+b[i]*x[i]+c[i]*x[i+1]=d[i]
+/** String \a how may contain:
+ * 'x', 'y', 'z' for solving along x-,y-,z-directions, or
+ * 'h' for solving along hexagonal direction at x-y plain (need nx=ny),
+ * 'c' for using periodical boundary conditions,
+ * 'd' for diffraction/diffuse calculation. */
+inline mglData mglTridMat(const mglDataA &A, const mglDataA &B, const mglDataA &C, const mglDataA &D, const char *how)
+{	return mglData(true, mgl_data_tridmat(&A, &B, &C, &D, how));	}
+//-----------------------------------------------------------------------------
 /// Calculate Jacobian determinant for D{x(u,v), y(u,v)} = dx/du*dy/dv-dx/dv*dy/du
 inline mglData mglJacobian(const mglDataA &x, const mglDataA &y)
 {	return mglData(true, mgl_jacobian_2d(&x, &y));	}
@@ -559,6 +542,9 @@ inline mglData mglIFS2d(const mglDataA &A, long n, long skip=20)
 /// Get array which is n-th points {x[i],y[i],z[i]} for iterated function system (fractal) generated by A
 inline mglData mglIFS3d(const mglDataA &A, long n, long skip=20)
 {	return mglData(true,mgl_data_ifs_3d(&A,n,skip));	}
+/// Get array which is n-th points {x[i],y[i],z[i]} for iterated function system (fractal) defined in *.ifs file 'fname' and named as 'name'
+inline mglData mglIFSfile(const char *fname, const char *name, long n, long skip=20)
+{	return mglData(true,mgl_data_ifs_file(fname,name,n,skip));	}
 //-----------------------------------------------------------------------------
 /// Get sub-array of the data with given fixed indexes
 inline mglData mglSubData(const mglDataA &dat, long xx, long yy=-1, long zz=-1)
@@ -882,10 +868,10 @@ public:
 	{	ind = i;	s = name;	}
 
 	/// Get the interpolated value and its derivatives in given data cell without border checking
-	mreal valueD(mreal x,mreal y=0,mreal z=0,mreal *dx=0,mreal *dy=0,mreal *dz=0) const
+	mreal valueD(mreal x,mreal y=0,mreal =0,mreal *dx=0,mreal *dy=0,mreal *dz=0) const
 	{	if(dz)	*dz=0;	return dat.valueD(ind,x,y,0,dx,dy);	}
 	/// Get the interpolated value in given data cell without border checking
-	mreal value(mreal x,mreal y=0,mreal z=0) const
+	mreal value(mreal x,mreal y=0,mreal =0) const
 	{	return dat.value(ind,x,y);	}
 	/// Get the value in given cell of the data without border checking
 	mreal v(long i,long j=0,long =0) const
@@ -929,10 +915,10 @@ public:
 	{	ind = i;	s = name;	}
 
 	/// Get the interpolated value and its derivatives in given data cell without border checking
-	mreal valueD(mreal x,mreal y=0,mreal z=0,mreal *dx=0,mreal *dy=0,mreal *dz=0) const
+	mreal valueD(mreal x,mreal =0,mreal =0,mreal *dx=0,mreal *dy=0,mreal *dz=0) const
 	{	if(dy)	*dy=0;	if(dz)	*dz=0;	return dat.valueD(x,ind,0,dx);	}
 	/// Get the interpolated value in given data cell without border checking
-	mreal value(mreal x,mreal y=0,mreal z=0) const
+	mreal value(mreal x,mreal =0,mreal =0) const
 	{	return dat.value(x,ind,0);	}
 	/// Get the value in given cell of the data without border checking
 	mreal v(long i,long =0,long =0) const
@@ -940,9 +926,9 @@ public:
 	mreal vthr(long i) const
 	{	return dat.vthr(i+dat.GetNx()*ind);	}
 	// add for speeding up !!!
-	mreal dvx(long i,long j=0,long =0) const
+	mreal dvx(long i,long =0,long =0) const
 	{	return	dat.dvx(i,ind,0);	}
-	mreal dvy(long i,long j=0,long =0) const
+	mreal dvy(long ,long =0,long =0) const
 	{	return 0;	}
 	mreal dvz(long ,long =0,long =0) const
 	{	return 0;	}
@@ -967,21 +953,21 @@ public:
 	const std::vector<mreal> &operator=(const std::vector<mreal> &st)	{	dat = st;	return st;	}
 
 	/// Get the interpolated value and its derivatives in given data cell without border checking
-	mreal valueD(mreal x,mreal y=0,mreal z=0,mreal *dx=0,mreal *dy=0,mreal *dz=0) const
+	mreal valueD(mreal x,mreal =0,mreal =0,mreal *dx=0,mreal *dy=0,mreal *dz=0) const
 	{	return mglSpline3(dat.data(),dat.size(),1,1,x,0,0,dx,dy,dz);	}
 	/// Get the interpolated value in given data cell without border checking
-	mreal value(mreal x,mreal y=0,mreal z=0) const
+	mreal value(mreal x,mreal =0,mreal =0) const
 	{	return mglSpline3s(dat.data(),dat.size(),1,1,x,0,0);	}
 
-	mreal v(long i,long j=0,long k=0) const		{	return dat[i];	}
+	mreal v(long i,long =0,long =0) const		{	return dat[i];	}
 	mreal vthr(long i) const	{	return dat[i];	};
 	long GetNx() const	{	return dat.size();	}
 	long GetNy() const	{	return 1;	}
 	long GetNz() const	{	return 1;	}
-	mreal dvx(long i,long j=0,long k=0) const
+	mreal dvx(long i,long =0,long =0) const
 	{	return i>0? (i<long(dat.size()-1)? (dat[i+1]-dat[i-1])/2:dat[i]-dat[i-1]) : dat[i+1]-dat[i];	}
-	mreal dvy(long i,long j=0,long k=0) const	{	return 1;	}
-	mreal dvz(long i,long j=0,long k=0) const	{	return 1;	}
+	mreal dvy(long ,long =0,long =0) const	{	return 1;	}
+	mreal dvz(long ,long =0,long =0) const	{	return 1;	}
 };
 //-----------------------------------------------------------------------------
 #endif
