@@ -1,6 +1,6 @@
 /***************************************************************************
  * datac.h is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -354,6 +354,9 @@ using mglDataA::Momentum;
 	 *  By default quadratic averaging over 5 points is used. */
 	inline void Smooth(const char *dirs="xyz",mreal delta=0)
 	{	mgl_datac_smooth(this,dirs,delta);	}
+	/// Limit the data to be inside [-v,v], keeping the original sign
+	inline void Limit(mreal v)
+	{	mgl_datac_limit(this, v);	}
 
 	/// Hankel transform
 	inline void Hankel(const char *dir)	{	mgl_datac_hankel(this,dir);	}
@@ -445,6 +448,9 @@ using mglDataA::Momentum;
 	/// Set the value in given cell of the data
 	void set_v(mreal val, long i,long j=0,long k=0)	{	mgl_datac_set_value(this,val,i,j,k);	}
 #endif
+	/// Get the complex value in given cell of the data
+	dual vc(long i,long j=0,long k=0) const	{	return a[i+nx*(j+ny*k)];	}
+	dual vcthr(long i) const	{	return a[i];	}
 	/// Get the interpolated value and its derivatives in given data cell without border checking
 	mreal valueD(mreal x,mreal y=0,mreal z=0,mreal *dx=0,mreal *dy=0,mreal *dz=0) const
 	{	dual aa,ax,ay,az;	mreal res;
@@ -482,6 +488,15 @@ inline mglDataC mglQO3dc(const char *ham, const mglDataA &ini_re, const mglDataA
 inline mglDataC mglQO3dc(const char *ham, const mglDataA &ini_re, const mglDataA &ini_im, const mglDataA &ray, mglData &xx, mglData &yy, mglData &zz, mreal r=1, mreal k0=100)
 {	return mglDataC(true, mgl_qo3d_solve_c(ham, &ini_re, &ini_im, &ray, r, k0, &xx, &yy, &zz));	}
 //-----------------------------------------------------------------------------
+/// Get array as solution of tridiagonal system of equations a[i]*x[i-1]+b[i]*x[i]+c[i]*x[i+1]=d[i]
+/** String \a how may contain:
+ * 'x', 'y', 'z' for solving along x-,y-,z-directions, or
+ * 'h' for solving along hexagonal direction at x-y plain (need nx=ny),
+ * 'c' for using periodical boundary conditions,
+ * 'd' for diffraction/diffuse calculation. */
+inline mglDataC mglTridMatC(const mglDataA &A, const mglDataA &B, const mglDataA &C, const mglDataC &D, const char *how)
+{	return mglDataC(true, mgl_datac_tridmat(&A, &B, &C, &D, how));	}
+//-----------------------------------------------------------------------------
 /// Get sub-array of the data with given fixed indexes
 inline mglDataC mglSubDataC(const mglDataA &dat, long xx, long yy=-1, long zz=-1)
 {	return mglDataC(true,mgl_datac_subdata(&dat,xx,yy,zz));	}
