@@ -1,6 +1,6 @@
 /***************************************************************************
  * define.h is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -20,6 +20,10 @@
 #ifndef _MGL_DEFINE_H_
 #define _MGL_DEFINE_H_
 //-----------------------------------------------------------------------------
+#if defined(_MSC_VER)
+#pragma warning(disable: 4996)
+#endif
+
 #include "mgl2/config.h"
 #ifndef SWIG
 
@@ -28,6 +32,7 @@
 #endif
 
 #include "mgl2/dllexport.h"
+
 #if MGL_HAVE_ATTRIBUTE
 #define MGL_FUNC_CONST	__attribute__((const))
 #define MGL_FUNC_PURE	__attribute__((pure))
@@ -267,6 +272,7 @@ extern uint64_t mgl_mask_val[16];
 #define MGL_PREFERVC 		0x040000 	///< Prefer vertex color instead of texture if output format supports
 #define MGL_ONESIDED 		0x080000 	///< Render only front side of surfaces if output format supports (for debugging)
 #define MGL_NO_ORIGIN 		0x100000 	///< Don't draw tick labels at axis origin
+#define MGL_GRAY_MODE 		0x100000 	///< Convert all colors to gray ones
 //-----------------------------------------------------------------------------
 #if MGL_HAVE_C99_COMPLEX
 #include <complex.h>
@@ -282,10 +288,26 @@ const mdual mgl_I=_Complex_I;
 #define mgl_abs(x)	cabs(x)
 #endif
 #ifdef __cplusplus
+#include <string>
+#include <vector>
+#if defined(_MSC_VER)
+template class MGL_EXPORT std::allocator<char>;
+template class MGL_EXPORT std::allocator<wchar_t>;
+template struct MGL_EXPORT std::char_traits<char>;
+template struct MGL_EXPORT std::char_traits<wchar_t>;
+template class MGL_EXPORT std::basic_string< char, std::char_traits<char>, std::allocator<char> >;
+template class MGL_EXPORT std::basic_string< wchar_t, std::char_traits<wchar_t>, std::allocator<wchar_t> >;
+template class MGL_EXPORT std::vector<long>;
+template class MGL_EXPORT std::vector<mreal>;
+#endif
 //-----------------------------------------------------------------------------
 extern float mgl_cos[360];	///< contain cosine with step 1 degree
 //-----------------------------------------------------------------------------
 #include <complex>
+#if defined(_MSC_VER)
+template class MGL_EXPORT std::complex<float>;
+template class MGL_EXPORT std::complex<double>;
+#endif
 typedef std::complex<mreal> dual;
 typedef std::complex<double> ddual;
 #if !MGL_HAVE_C99_COMPLEX
