@@ -1,6 +1,6 @@
 /***************************************************************************
  * parser.h is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -22,7 +22,6 @@
 
 #ifdef __cplusplus
 #include "mgl2/mgl.h"
-#include <string>
 #if MGL_HAVE_LTDL
 #include <ltdl.h>
 #endif
@@ -161,6 +160,8 @@ public:
 	void DeleteVar(const wchar_t *name);
 	/// Delete all data variables
 	void DeleteAll();
+	/// Set variant of argument(s) separated by '?' to be used
+	inline void SetVariant(int var=0)	{	Variant = var<=0?0:var;	}
 private:
 //	long parlen;		///< Length of parameter strings
 	std::wstring par[40];	///< Parameter for substituting instead of $1, ..., $9
@@ -176,6 +177,7 @@ private:
 	int for_stack[40];	///< The order of for-variables
 	int for_addr;		///< Flag for saving address in variable (for_addr-1)
 	bool for_br;		///< Break is switched on (skip all comands until 'next')
+	unsigned Variant;	///< Select variant of argument(s) separated by '?'
 
 	/// Parse command
 	int Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const std::wstring &var, const wchar_t *opt);
