@@ -1,6 +1,6 @@
 /***************************************************************************
  * type.h is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -93,6 +93,8 @@ inline bool operator>(const mglPoint &a, const mglPoint &b)
 {	return a.x>=b.x && a.y>=b.y && a.z>=b.z;	}
 inline mreal mgl_norm(const mglPoint &p)
 {	return sqrt(p.x*p.x+p.y*p.y+p.z*p.z);	}
+inline mreal mgl_anorm(const mglPoint &p)
+{	return fabs(p.x)+fabs(p.y)+fabs(p.z);	}
 #endif
 //-----------------------------------------------------------------------------
 /// Class for RGBA color
