@@ -1,6 +1,6 @@
 /***************************************************************************
  * base.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -20,6 +20,28 @@
 #include "mgl2/font.h"
 #include "mgl2/base.h"
 #include "mgl2/eval.h"
+#if MGL_HAVE_OMP
+#include <omp.h>
+#endif
+
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_mutex_unlock(void *mutex)
+{
+#if MGL_HAVE_PTHREAD
+	pthread_mutex_unlock((pthread_mutex_t *)mutex);
+#elif MGL_HAVE_OMP
+	omp_unset_lock((omp_lock_t *)mutex);
+#endif
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_mutex_lock(void *mutex)
+{
+#if MGL_HAVE_PTHREAD
+	pthread_mutex_lock((pthread_mutex_t *)mutex);
+#elif MGL_HAVE_OMP
+	omp_set_lock((omp_lock_t *)mutex);
+#endif
+}
 //-----------------------------------------------------------------------------
 char *mgl_strdup(const char *s)
 {
@@ -105,13 +127,16 @@ mglBase::mglBase()
 	pthread_mutex_init(&mutexClf,0);
 	Pnt.set_mutex(&mutexClf);
 	Prm.set_mutex(&mutexClf);
-	Txt.set_mutex(&mutexClf);
+//	Txt.set_mutex(&mutexClf);
 #endif
 #if MGL_HAVE_OMP
-	omp_init_lock(&lockClf);
-	Pnt.set_mutex(&lockClf);
-	Prm.set_mutex(&lockClf);
-	Txt.set_mutex(&lockClf);
+	lockClf = new omp_lock_t;
+	omp_init_lock((omp_lock_t*)lockClf);
+	Pnt.set_mutex(lockClf);
+	Prm.set_mutex(lockClf);
+//	Txt.set_mutex(&lockClf);
+#else
+	lockClf = NULL;
 #endif
 	fnt=0;	*FontDef=0;	fx=fy=fz=fa=fc=0;
 	AMin.Set(0,0,0,0);	AMax.Set(1,1,1,1);
@@ -127,12 +152,28 @@ mglBase::mglBase()
 	MinS.Set(-1,-1,-1);	MaxS.Set(1,1,1);
 	fnt = new mglFont;	fnt->gr = this;	PrevState=NAN;	size_opt=NAN;
 }
+//-----------------------------------------------------------------------------
 mglBase::~mglBase()
 {
 	ClearEq();	ClearPrmInd();	delete fnt;
-	Pnt.set_mutex(0);	Prm.set_mutex(0);	Txt.set_mutex(0);
+	Pnt.set_mutex(0);	Prm.set_mutex(0);	//Txt.set_mutex(0);
+#if MGL_HAVE_PTHREAD
+	pthread_mutex_destroy(&mutexPnt);
+	pthread_mutex_destroy(&mutexTxt);
+	pthread_mutex_destroy(&mutexSub);
+	pthread_mutex_destroy(&mutexLeg);
+	pthread_mutex_destroy(&mutexPrm);
+	pthread_mutex_destroy(&mutexPtx);
+	pthread_mutex_destroy(&mutexStk);
+	pthread_mutex_destroy(&mutexGrp);
+	pthread_mutex_destroy(&mutexGlf);
+	pthread_mutex_destroy(&mutexAct);
+	pthread_mutex_destroy(&mutexDrw);
+	pthread_mutex_destroy(&mutexClf);
+#endif
 #if MGL_HAVE_OMP
-	omp_destroy_lock(&lockClf);
+	omp_destroy_lock((omp_lock_t*)lockClf);
+	delete ((omp_lock_t*)lockClf);
 #endif
 }
 //-----------------------------------------------------------------------------
@@ -299,6 +340,11 @@ long mglBase::AddPnt(const mglMatrix *mat, mglPoint p, mreal c, mglPoint n, mrea
 	if(ci<0 || ci>=(long)Txt.size())	ci=0;	// NOTE never should be here!!!
 	const mglTexture &txt=Txt[ci];
 	txt.GetC(c,a,q);	// RGBA color
+	if(get(MGL_GRAY_MODE))
+	{
+		float h = 0.3*q.r + 0.59*q.g + 0.11*q.b;
+		q.r = q.g = q.b = h;
+	}
 
 	// add gap for texture coordinates for compatibility with OpenGL
 	const mreal gap = 0./MGL_TEXTURE_COLOURS;
