@@ -1,6 +1,6 @@
 /***************************************************************************
  * pde.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -29,12 +29,160 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(const char *str, const std::vector<mglDataA*>
 //		Advanced PDE series in 2D case
 //
 //-----------------------------------------------------------------------------
+void MGL_NO_EXPORT mgl_operator_exp(long n, dual *h, dual *a, dual *f)
+{
+	memset(f,0,2*n*sizeof(dual));
+	const long i1=n/2, i2=3*n/2-1;
+#pragma omp parallel for
+	for(long j=0;j<n;j++)
+	{
+		long jp = (j+1)%n;
+		mreal h1=real(h[n*j]), h2=real(h[n-1+n*j]);
+		mreal g1=(h1+real(h[n*jp]))/2, g2=(h2+real(h[n-1+n*jp]))/2;
+		mreal k1=M_PI*2*j/n, k2 = M_PI*(2*j+1)/n;
+		for(long i=0;i<i1;i++)
+		{
+			f[2*j] += a[i]*exp(dual(0,h1+i*k1));
+			f[2*j+1] += a[i]*exp(dual(0,g1+i*k2));
+		}
+		for(long i=i1;i<i2;i++)
+		{
+			mreal hh = real(h[i-i1+n*j]);
+			f[2*j] += a[i]*exp(dual(0,hh+i*k1));
+			f[2*j+1] += a[i]*exp(dual(0,(hh+real(h[i-i1+n*jp]))/2+i*k2));
+		}
+		for(long i=i2;i<2*n;i++)
+		{
+			f[2*j] += a[i]*exp(dual(0,h2+i*k1));
+			f[2*j+1] += a[i]*exp(dual(0,g2+i*k2));
+		}
+	}
+	memset(a,0,2*n*sizeof(dual));
+#pragma omp parallel for
+	for(long i=0;i<2*n;i++)
+	{
+		register long ii=i-i1;
+		if(ii<0)	ii=0;	if(ii>n-1)	ii=n-1;
+		double kk=M_PI*2*i/n;
+		for(long j=0;j<n;j++)
+		{
+			mreal h1 = real(h[ii+n*j]), g1 = (h1+real(h[ii+n*((j+1)%n)]))/2;
+			a[i] += f[2*j]*exp(dual(0,h1-kk*j));
+			a[i] += f[2*j+1]*exp(dual(0,g1-kk*(j+0.5)));
+		}
+	}
+}
+//-----------------------------------------------------------------------------
+void MGL_NO_EXPORT mgl_operator_hrm(long n, dual *h, dual *a, dual *f, dual *g, dual *o)
+{
+	memset(f,0,2*n*sizeof(dual));
+	memset(g,0,2*n*sizeof(dual));
+	const long i1=n/2, i2=3*n/2-1;
+	#pragma omp parallel for
+	for(long j=0;j<n;j++)
+	{
+		long jp = (j+1)%n;
+		mreal h1=real(h[n*j]), h2=real(h[n-1+n*j]);
+		mreal g1=h1+real(h[n*jp])/2., g2=h2+real(h[n-1+n*jp])/2.;
+		mreal k1=M_PI*2*j/n, k2 = M_PI*(2*j+1)/n;
+		for(long i=0;i<i1;i++)
+		{
+			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
+			f[2*j] += a[i]*h1*e1;	f[2*j+1] += a[i]*g1*e2;
+			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
+		}
+		for(long i=i1;i<i2;i++)
+		{
+			mreal hh = real(h[i-i1+n*j]);
+			mreal gg = hh+real(h[i-i1+n*jp])/2.;
+			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
+			f[2*j] += a[i]*hh*e1;	f[2*j+1] += a[i]*gg*e2;
+			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
+		}
+		for(long i=i2;i<2*n;i++)
+		{
+			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
+			f[2*j] += a[i]*h2*e1;	f[2*j+1] += a[i]*g2*e2;
+			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
+		}
+	}
+	memset(o,0,2*n*sizeof(dual));
+	#pragma omp parallel for
+	for(long i=0;i<2*n;i++)
+	{
+		register long ii=i-i1;
+		if(ii<0)	ii=0;	if(ii>n-1)	ii=n-1;
+		double kk=M_PI*2*i/n;
+		for(long j=0;j<n;j++)
+		{
+			mreal h1 = real(h[ii+n*j]);
+			mreal g1 = h1+real(h[ii+n*((j+1)%n)])/2.;
+			dual e1=exp(dual(0,-j*kk)), e2=exp(dual(0,-kk*(j+0.5)));
+			o[i] += f[2*j]*e1 + f[2*j+1]*e2;
+			o[i] += g[2*j]*h1*e1 + g[2*j+1]*g1*e2;
+		}
+	}
+}
+//-----------------------------------------------------------------------------
+inline dual isqrt(dual a)
+{	mreal b=imag(a);	return b>=0?dual(sqrt(b),0):dual(0,sqrt(-b));	}
+void MGL_NO_EXPORT mgl_operator_lin(long n, dual *h, dual *a, dual *f, dual *g, dual *o)
+{
+	memset(f,0,2*n*sizeof(dual));
+	memset(g,0,2*n*sizeof(dual));
+	const long i1=n/2, i2=3*n/2-1;
+#pragma omp parallel for
+	for(long j=0;j<n;j++)
+	{
+		long jp = (j+1)%n;
+		dual h1=tanh(isqrt(h[n*j])), h2=tanh(isqrt(h[n-1+n*j]));
+		dual g1=tanh(h1+isqrt(h[n*jp]))/2., g2=tanh(h2+isqrt(h[n-1+n*jp]))/2.;
+		mreal k1=M_PI*2*j/n, k2 = M_PI*(2*j+1)/n;
+		for(long i=0;i<i1;i++)
+		{
+			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
+			f[2*j] += a[i]*h1*e1;	f[2*j+1] += a[i]*g1*e2;
+			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
+		}
+		for(long i=i1;i<i2;i++)
+		{
+			dual hh = tanh(isqrt(h[i-i1+n*j]));
+			dual gg = tanh(hh+isqrt(h[i-i1+n*jp]))/2.;
+			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
+			f[2*j] += a[i]*hh*e1;	f[2*j+1] += a[i]*gg*e2;
+			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
+		}
+		for(long i=i2;i<2*n;i++)
+		{
+			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
+			f[2*j] += a[i]*h2*e1;	f[2*j+1] += a[i]*g2*e2;
+			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
+		}
+	}
+	memset(o,0,2*n*sizeof(dual));
+#pragma omp parallel for
+	for(long i=0;i<2*n;i++)
+	{
+		register long ii=i-i1;
+		if(ii<0)	ii=0;	if(ii>n-1)	ii=n-1;
+		double kk=M_PI*2*i/n;
+		for(long j=0;j<n;j++)
+		{
+			dual h1 = tanh(isqrt(h[ii+n*j]));
+			dual g1 = tanh(h1+isqrt(h[ii+n*((j+1)%n)]))/2.;
+			dual e1=exp(dual(0,-j*kk)), e2=exp(dual(0,-kk*(j+0.5)));
+			o[i] += f[2*j]*e1 + f[2*j+1]*e2;
+			o[i] += g[2*j]*h1*e1 + g[2*j+1]*g1*e2;
+		}
+	}
+}
+//-----------------------------------------------------------------------------
 // Solve equation dx/dy = func(p,x,y,|u|)[u] where p=d/dx. There are no assumptions about form of func().
 HADT MGL_EXPORT mgl_pde_adv_c(HMGL gr, const char *func, HCDT ini_re, HCDT ini_im, mreal dt, mreal k0, const char *opt)
 {
 	mreal gamma = gr->SaveState(opt);	if(mgl_isnan(gamma))	gamma = 20;
-	mglPoint Min=gr->Min, Max=gr->Max;
-	long nx=ini_re->GetNx(), nt = long((Max.y-Min.y)/dt)+1;
+	const mglPoint &Min=gr->Min, &Max=gr->Max;
+	const long nx=ini_re->GetNx(), nt = long((Max.y-Min.y)/dt)+1;
 	if(nx<2 || nt<2 || Max.x==Min.x){	gr->SetWarn(mglWarnLow,"PDE");	return 0;	}	// Too small data
 	if(ini_im->GetNx() != nx)		{	gr->SetWarn(mglWarnDim,"PDE");	return 0;	}	// Wrong dimensions
 
@@ -49,7 +197,7 @@ HADT MGL_EXPORT mgl_pde_adv_c(HMGL gr, const char *func, HCDT ini_re, HCDT ini_i
 	list.push_back(&x);	list.push_back(&y);	list.push_back(&p);	list.push_back(&r);	list.push_back(&u);
 
 	dual *a = new dual[2*nx];	memset(a,0,2*nx*sizeof(dual));	// Add "damping" area
-	dual *f = new dual[2*nx];	memset(f,0,2*nx*sizeof(dual));	// Effective "spectrum"
+	dual *f = new dual[6*nx], *g=f+2*nx, *s=f+4*nx;
 #pragma omp parallel for
 	for(long i=0;i<nx;i++)	// Initial conditions
 		a[i+nx/2] = dual(ini_re->v(i), ini_im->v(i));
@@ -60,57 +208,26 @@ HADT MGL_EXPORT mgl_pde_adv_c(HMGL gr, const char *func, HCDT ini_re, HCDT ini_i
 		if(i<nx/2)		dmp[i] += gamma*mgl_ipow((nx/2-i)/mreal(nx/2),2);
 		if(i>3*nx/2)	dmp[i] += gamma*mgl_ipow((i-3*nx/2-1)/mreal(nx/2),2);
 	}
+	bool have_y = mglchr(func,'y');
 	HADT ham;
+	if(!have_y)
+	{	ham = mglFormulaCalcC(func, list);	mgl_datac_mul_num(ham,dd);	}
 	for(long k=0;k<nt;k++)
 	{
-		y.Fill(k*dt);
 		memcpy(u.a,a+nx/2,nx*sizeof(dual));
 		memcpy(res->a+k*nx,a+nx/2,nx*sizeof(dual));
-		ham = mglFormulaCalcC(func, list);
-		memset(f,0,2*nx*sizeof(dual));
-		const long i1=nx/2, i2=3*nx/2-1;
-#pragma omp parallel for
-		for(long j=0;j<nx;j++)
-		{
-			long jp = j<nx-1?j+1:0;
-			dual h1=dual(0,dd)*ham->a[nx*j], h2=dual(0,dd)*ham->a[nx-1+nx*j];
-			dual g1=(h1+dual(0,dd)*ham->a[nx*jp])/mreal(2), g2=(h2+dual(0,dd)*ham->a[nx-1+nx*jp])/mreal(2);
-			mreal k1=M_PI*2*j/nx, k2 = M_PI*(2*j+1)/nx;
-			for(long i=0;i<i1;i++)
-			{
-				f[2*j] += a[i]*exp(h1+dual(0,i*k1));
-				f[2*j+1] += a[i]*exp(g1+dual(0,i*k2));
-			}
-			for(long i=i1;i<i2;i++)
-			{
-				dual hh = ham->a[i-i1+nx*j];
-				f[2*j] += a[i]*exp(dual(0,dd)*hh+dual(0,i*k1));
-				f[2*j+1] += a[i]*exp(dual(0,dd)*(hh+ham->a[i-i1+nx*jp])/mreal(2)+dual(0,i*k2));
-			}
-			for(long i=i2;i<2*nx;i++)
-			{
-				f[2*j] += a[i]*exp(h2+dual(0,i*k1));
-				f[2*j+1] += a[i]*exp(g2+dual(0,i*k2));
-			}
-		}
-		memset(a,0,2*nx*sizeof(dual));
+		if(have_y)
+		{	y.Fill(k*dt);	ham = mglFormulaCalcC(func, list);	mgl_datac_mul_num(ham,dd);	}
+		mgl_operator_exp(nx,ham->a,a,f);
+		mgl_operator_lin(nx,ham->a,a,f,g,s);
+		mgl_operator_lin(nx,ham->a,s,f,g,s);
 #pragma omp parallel for
 		for(long i=0;i<2*nx;i++)
-		{
-			register long ii=i-i1;
-			if(ii<0)	ii=0;	if(ii>nx-1)	ii=nx-1;
-			double kk=M_PI*2*i/nx;
-			for(long j=0;j<nx;j++)
-			{
-				dual h1 = ham->a[ii+nx*j], g1 = (h1+ham->a[ii+nx*(j<nx-1?j+1:0)])/mreal(2);
-				a[i] += f[2*j]*exp(dual(0,dd)*h1-dual(0,kk*j));
-				a[i] += f[2*j+1]*exp(dual(0,dd)*g1-dual(0,kk*(j+0.5)));
-			}
-			a[i] *= exp(-dmp[i]*dt)/mreal(2*nx);
-		}
-		delete ham;
+			a[i] = (a[i]-s[i]/mreal(8*nx*nx))*exp(-dmp[i]*dt)/mreal(2*nx);
+		if(have_y)	delete ham;
 	}
 	delete []a;	delete []f;	delete []dmp;
+	if(!have_y)	delete ham;
 	gr->LoadState();	return res;
 }
 //-----------------------------------------------------------------------------
@@ -195,7 +312,7 @@ struct mgl_pde_ham
 };
 void MGL_NO_EXPORT mgl_pde_hprep(const mgl_pde_ham *f)
 {
-	long nx = f->nx, ny = f->ny;
+	const long nx = f->nx, ny = f->ny;
 	mglDataV x(nx,ny), y(nx,ny), z, r(nx,ny);
 	mglDataW p(nx,ny), q(nx,ny);
 	x.s = L"x";	y.s = L"y";	p.s = L"p";	q.s = L"q";	r.s=L"#$mgl";
@@ -420,7 +537,7 @@ HMDT MGL_EXPORT mgl_ode_solve(void (*func)(const mreal *x, mreal *dx, void *par)
 HMDT MGL_EXPORT mgl_ode_solve_ex(void (*func)(const mreal *x, mreal *dx, void *par), int n, const mreal *x0, mreal dt, mreal tmax, void *par, void (*bord)(mreal *x, const mreal *xp, void *par))
 {
 	if(tmax<dt)	return 0;	// nothing to do
-	int nt = int(tmax/dt+0.5)+1;
+	const long nt = int(tmax/dt+0.5)+1;
 	mglData *res=new mglData(n,nt);
 	mreal *x=new mreal[n], *k1=new mreal[n], *k2=new mreal[n], *k3=new mreal[n], *v=new mreal[n], hh=dt/2;
 	register long i,k;
@@ -481,7 +598,7 @@ struct mgl_ap
 	mgl_ap()	{	memset(this,0,sizeof(mgl_ap));	}
 };
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_init_ra(int n, int n7, const mreal *r, mgl_ap *ra)	// prepare some intermediate data for QO (3d case)
+void MGL_NO_EXPORT mgl_init_ra(long n, int n7, const mreal *r, mgl_ap *ra)	// prepare some intermediate data for QO (3d case)
 {
 	register double tt;
 	tt = hypot(r[n7]-r[0], r[n7+1]-r[1]);
@@ -498,34 +615,35 @@ void MGL_NO_EXPORT mgl_init_ra(int n, int n7, const mreal *r, mgl_ap *ra)	// pre
 	ra[0].x2 = ra[0].y1*ra[0].z0 - ra[0].y0*ra[0].z1;	// vector g_2
 	ra[0].y2 = ra[0].z1*ra[0].x0 - ra[0].z0*ra[0].x1;
 	ra[0].z2 = ra[0].x1*ra[0].y0 - ra[0].x0*ra[0].y1;
-	register long i;
-	for(i=1;i<n;i++)	// NOTE: no parallel due to dependence on prev point!
+	for(long i=1;i<n;i++)	// NOTE: no parallel due to dependence on prev point!
 	{
-		ra[i].dt = r[6+n7*i] - r[6+n7*(i-1)];
-		ra[i].x0 = r[n7*i]   - r[n7*(i-1)];		// NOTE: very rough formulas
-		ra[i].y0 = r[n7*i+1] - r[n7*(i-1)+1];	// for corresponding with dt one
-		ra[i].z0 = r[n7*i+2] - r[n7*(i-1)+2];	// for corresponding with dt one
-		tt = sqrt(ra[i].x0*ra[i].x0 + ra[i].y0*ra[i].y0 + ra[i].z0*ra[i].z0);
-		ra[i].x0 /= tt;	ra[i].y0 /= tt;	ra[i].z0 /= tt;
-		ra[i].ch = tt/ra[i].dt;
-		tt = ra[i].x0*ra[i-1].x1 + ra[i].y0*ra[i-1].y1 + ra[i].z0*ra[i-1].z1;
-		ra[i].x1 = ra[i-1].x1 - tt*ra[i].x0;	// vector g_1
-		ra[i].y1 = ra[i-1].y1 - tt*ra[i].y0;
-		ra[i].z1 = ra[i-1].z1 - tt*ra[i].z0;
-		ra[i].t1 = tt/(ra[i].dt*ra[i].ch);
-		tt = sqrt(ra[i].x1*ra[i].x1 + ra[i].y1*ra[i].y1 + ra[i].z1*ra[i].z1);
-		ra[i].x1 /= tt;	ra[i].y1 /= tt;	ra[i].z1 /= tt;	// norm for reducing numeric error
-		ra[i].x2 = ra[i].y1*ra[i].z0 - ra[i].y0*ra[i].z1;	// vector g_2
-		ra[i].y2 = ra[i].z1*ra[i].x0 - ra[i].z0*ra[i].x1;
-		ra[i].z2 = ra[i].x1*ra[i].y0 - ra[i].x0*ra[i].y1;
-		tt = ra[i].x0*ra[i-1].x2 + ra[i].y0*ra[i-1].y2 + ra[i].z0*ra[i-1].z2;
-		ra[i].t2 = tt/(ra[i].dt*ra[i].ch);
-		// other parameters
-		ra[i].pt = r[n7*i+3]*ra[i].x0 + r[n7*i+4]*ra[i].y0 + r[n7*i+5]*ra[i].z0;
-		ra[i].q1 = r[n7*i+3]*ra[i].x1 + r[n7*i+4]*ra[i].y1 + r[n7*i+5]*ra[i].z1;
-		ra[i].q2 = r[n7*i+3]*ra[i].x2 + r[n7*i+4]*ra[i].y2 + r[n7*i+5]*ra[i].z2;
-		ra[i].d1 = (ra[i].q1-ra[i-1].q1)/(ra[i].dt*ra[i].ch);
-		ra[i].d2 = (ra[i].q2-ra[i-1].q2)/(ra[i].dt*ra[i].ch);
+		mgl_ap *ri=ra+i, *rp=ra+i-1;
+		const mreal *rr = r+n7*i;
+		ri->dt = rr[6] - rr[6-n7];
+		ri->x0 = rr[0] - rr[-n7];	// NOTE: very rough formulas
+		ri->y0 = rr[1] - rr[1-n7];	// for corresponding with dt one
+		ri->z0 = rr[2] - rr[2-n7];	// for corresponding with dt one
+		double ch = sqrt(ri->x0*ri->x0 + ri->y0*ri->y0 + ri->z0*ri->z0);
+		ri->x0 /= ch;	ri->y0 /= ch;	ri->z0 /= ch;
+		ri->ch = ch/ri->dt;
+		ri->pt = rr[3]*ri->x0 + rr[4]*ri->y0 + rr[5]*ri->z0;
+		ri->q1 = rr[3]*ri->x1 + rr[4]*ri->y1 + rr[5]*ri->z1;
+		ri->q2 = rr[3]*ri->x2 + rr[4]*ri->y2 + rr[5]*ri->z2;
+		// NOTE previous point is used here!
+		tt = ri->x0*rp->x1 + ri->y0*rp->y1 + ri->z0*rp->z1;
+		ri->x1 = rp->x1 - tt*ri->x0;	// vector g_1
+		ri->y1 = rp->y1 - tt*ri->y0;
+		ri->z1 = rp->z1 - tt*ri->z0;
+		ri->t1 = tt/ch;
+		tt = sqrt(ri->x1*ri->x1 + ri->y1*ri->y1 + ri->z1*ri->z1);
+		ri->x1 /= tt;	ri->y1 /= tt;	ri->z1 /= tt;	// norm for reducing numeric error
+		ri->x2 = ri->y1*ri->z0 - ri->y0*ri->z1;	// vector g_2
+		ri->y2 = ri->z1*ri->x0 - ri->z0*ri->x1;
+		ri->z2 = ri->x1*ri->y0 - ri->x0*ri->y1;
+		tt = ri->x0*rp->x2 + ri->y0*rp->y2 + ri->z0*rp->z2;
+		ri->t2 = tt/ch;
+		ri->d1 = (ri->q1-rp->q1)/ch;
+		ri->d2 = (ri->q2-rp->q2)/ch;
 	}
 	memcpy(ra,ra+1,sizeof(mgl_ap));	// setup zero point
 	ra[0].pt = r[3]*ra[0].x0 + r[4]*ra[0].y0 + r[5]*ra[0].z0;
@@ -554,7 +672,7 @@ MGL_NO_EXPORT void *mgl_qo2d_hprep(void *par)
 	mgl_ap *ra = f->ra;
 
 	const mreal *r = f->r;
-	long nx=t->n;
+	const long nx=t->n;
 #if !MGL_HAVE_PTHREAD
 #pragma omp parallel for
 #endif
@@ -581,7 +699,7 @@ HADT MGL_EXPORT mgl_qo2d_func_c(ddual (*ham)(mreal u, mreal x, mreal y, mreal px
 {
 	const mglData *ray=dynamic_cast<const mglData *>(ray_dat);	// NOTE: Ray must be mglData!
 	if(!ray)	return 0;
-	long nx=ini_re->GetNx(), nt=ray->ny, n7=ray->nx;
+	const long nx=ini_re->GetNx(), nt=ray->ny, n7=ray->nx;
 	if(nx<2 || ini_im->GetNx()!=nx || nt<2)	return 0;
 	mglDataC *res=new mglDataC(nx,nt,1);
 
@@ -706,7 +824,7 @@ MGL_NO_EXPORT void *mgl_qo3d_hprep(void *par)
 	mgl_qo3d_ham *f = (mgl_qo3d_ham *)t->v;
 	mgl_ap *ra = f->ra;
 	const mreal *r = f->r;
-	long nx=t->n;
+	const long nx=t->n;
 #if !MGL_HAVE_PTHREAD
 #pragma omp parallel for
 #endif
@@ -739,7 +857,7 @@ MGL_NO_EXPORT void *mgl_qo3d_post(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	mgl_qo3d_ham *f = (mgl_qo3d_ham *)t->v;
-	long nx=t->n;
+	const long nx=t->n;
 #if !MGL_HAVE_PTHREAD
 #pragma omp parallel for
 #endif
@@ -764,7 +882,7 @@ HADT MGL_EXPORT mgl_qo3d_func_c(ddual (*ham)(mreal u, mreal x, mreal y, mreal z,
 {
 	const mglData *ray=dynamic_cast<const mglData *>(ray_dat);	// NOTE: Ray must be mglData!
 	if(!ray)	return 0;
-	long nx=ini_re->GetNx(), nt=ray->ny, n7=ray->nx;	// NOTE: only square grids are supported now (for simplicity)
+	const long nx=ini_re->GetNx(), nt=ray->ny, n7=ray->nx;	// NOTE: only square grids are supported now (for simplicity)
 	if(nx<2 || ini_re->GetNx()!=nx || ini_im->GetNx()*ini_im->GetNy()!=nx*nx || nt<2)	return 0;
 	mglDataC *res=new mglDataC(nx,nx,nt);
 
@@ -915,7 +1033,7 @@ uintptr_t MGL_EXPORT mgl_qo3d_solve_(const char *ham, uintptr_t* ini_re, uintptr
 MGL_NO_EXPORT void *mgl_jacob2(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
-	long nx=t->p[0], ny=t->p[1];
+	const long nx=t->p[0], ny=t->p[1];
 	mreal *r=t->a;
 	const mreal *x=t->b, *y=t->c;
 #if !MGL_HAVE_PTHREAD
@@ -934,7 +1052,7 @@ MGL_NO_EXPORT void *mgl_jacob2(void *par)
 }
 HMDT MGL_EXPORT mgl_jacobian_2d(HCDT x, HCDT y)
 {
-	int nx = x->GetNx(), ny=x->GetNy();
+	const long nx = x->GetNx(), ny=x->GetNy();
 	if(nx!=y->GetNx() || ny!=y->GetNy() || nx<2 || ny<2)	return	0;
 	mglData *r=new mglData(nx,ny,1);
 	const mglData *xx=dynamic_cast<const mglData *>(x);
@@ -962,7 +1080,7 @@ HMDT MGL_EXPORT mgl_jacobian_2d(HCDT x, HCDT y)
 MGL_NO_EXPORT void *mgl_jacob3(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
-	long nx=t->p[0], ny=t->p[1], nz=t->p[2];
+	const long nx=t->p[0], ny=t->p[1], nz=t->p[2];
 	mreal *r=t->a;
 	const mreal *x=t->b, *y=t->c, *z=t->d;
 #if !MGL_HAVE_PTHREAD
@@ -985,7 +1103,7 @@ MGL_NO_EXPORT void *mgl_jacob3(void *par)
 }
 HMDT MGL_EXPORT mgl_jacobian_3d(HCDT x, HCDT y, HCDT z)
 {
-	int nx = x->GetNx(), ny=x->GetNy(), nz=x->GetNz(), nn = nx*ny*nz;
+	const long nx = x->GetNx(), ny=x->GetNy(), nz=x->GetNz(), nn = nx*ny*nz;
 	if(nx<2 || ny<2 || nz<2)	return 0;
 	if(nn!=y->GetNN() || nn!=z->GetNN())	return 0;
 	mglData *r=new mglData(nx,ny,nz);
@@ -1024,3 +1142,327 @@ uintptr_t MGL_EXPORT mgl_jacobian_2d_(uintptr_t* x, uintptr_t* y)
 uintptr_t MGL_EXPORT mgl_jacobian_3d_(uintptr_t* x, uintptr_t* y, uintptr_t* z)
 {	return uintptr_t(mgl_jacobian_3d(_DA_(x), _DA_(y), _DA_(z)));	}
 //-----------------------------------------------------------------------------
+//
+//	Progonka
+//
+//-----------------------------------------------------------------------------
+void MGL_NO_EXPORT mgl_progonka_sr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long i0, long di, bool difr)
+{
+	mreal *aa=dat, *bb=dat+n, *uu=dat+2*n;
+	mreal b0=B->vthr(i0), c0=C->vthr(i0), d0=D->vthr(i0);
+	if(difr)	d0 = (2.-b0)*d0-c0*D->vthr(i0+di);
+	aa[0] = -c0/b0;	bb[0] = d0/b0;
+	for(long i=1;i<n;i++)
+	{
+		register long ii=i0+di*i;
+		mreal a=A->vthr(ii), b=B->vthr(ii), c=C->vthr(ii);
+		mreal d=difr?-a*D->vthr(ii-di)+(2.-b)*D->vthr(ii)-c*D->vthr(ii+di):D->vthr(ii);
+		aa[i] = -c/(b+a*aa[i-1]);
+		bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
+	}
+	uu[n-1] = bb[n-1];
+	for(long i=n-2;i>=0;i--)	uu[i] = bb[i]+aa[i]*uu[i+1];
+}
+void MGL_NO_EXPORT mgl_progonka_pr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long i0, long di, bool difr)
+{
+	mreal *aa=dat, *bb=dat+n, *gg=dat+2*n, *uu=dat+3*n;
+	mreal a0=A->vthr(i0), b0=B->vthr(i0), c0=C->vthr(i0), d0=D->vthr(i0);
+	if(difr)	d0 = -a0*D->vthr(i0+di*(n-1))+(2.-b0)*d0-c0*D->vthr(i0+di);
+	aa[0] =-c0/b0;	bb[0] = d0/b0;	gg[0] =-a0/b0;
+	for(long i=1;i<n;i++)
+	{
+		register long ii=i0+di*i, il=i0+di*((i+1)%n);
+		mreal a=A->vthr(ii), b=B->vthr(ii), c=C->vthr(ii);
+		mreal d=difr?-a*D->vthr(ii-di)+(2.-b)*D->vthr(ii)-c*D->vthr(il):D->vthr(ii);
+		aa[i] = -c/(b+a*aa[i-1]);
+		bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
+		gg[i] = -a*gg[i-1]/(b+a*aa[i-1]);
+	}
+	mreal P=bb[n-1]/(1.-gg[n-1]), Q=aa[n-1]/(1.-gg[n-1]);
+	aa[n-1] = Q;	bb[n-1] = P;
+	for(long i=n-2;i>=0;i--)
+	{
+		bb[i] += aa[i]*bb[i+1]+gg[i]*P;
+		aa[i] = aa[i]*aa[i+1]+gg[i]*Q;
+	}
+	mreal u0 = bb[0]/(1.-aa[0]);
+	for(long i=0;i<n;i++)	uu[i]=bb[i]+aa[i]*u0;
+}
+void MGL_NO_EXPORT mgl_progonka_hr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long i0, bool difr)
+{
+	mreal *aa=dat, *bb=dat+n, *uu=dat+n*n;
+	mreal b0=B->vthr(i0), c0=C->vthr(i0), d0=D->vthr(i0);
+	uu[0] = d0/b0*(difr?(2.-b0):1.);
+	b0=B->vthr(i0+n*n-1);	d0=D->vthr(i0+n*n-1);
+	uu[n*n-1] = d0/b0*(difr?(2.-b0):1.);
+	long di = n-1, i1 = i0+n*(n-1);
+	// suppose the square grid!
+	for(long j=1;j<n;j++)
+	{
+		// first bottom-left triangle
+		b0=B->vthr(i0+j);	c0=C->vthr(i0+j);	d0=D->vthr(i0+j);
+		if(difr)	d0 = (2.-b0)*d0-c0*D->vthr(i0+j+di);
+		aa[0] = -c0/b0;	bb[0] = d0/b0;
+		for(long i=1;i<=j;i++)
+		{
+			register long ii=i0+j+di*i;
+			mreal a=A->vthr(ii),b=B->vthr(ii),c=C->vthr(ii);
+			mreal d=difr?-a*D->vthr(ii-di)+(2.-b)*D->vthr(ii)-c*D->vthr(ii+di):D->vthr(ii);
+			aa[i] = -c/(b+a*aa[i-1]);
+			bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
+		}
+		uu[j+di*(j-1)] = bb[j];
+		for(long i=j-1;i>=0;i--)
+			uu[j+di*i] = bb[i]+aa[i]*uu[j+di*i+di];
+		// next top-right triangle
+		long j1=n-1-j;
+		b0=B->vthr(i1+j1);	c0=C->vthr(i1+j1);	d0=D->vthr(i1+j1);
+		if(difr)	d0 = (2.-b0)*d0-c0*D->vthr(i1+j1-di);
+		aa[0] = -c0/b0;	bb[0] = d0/b0;
+		for(long i=1;i<=j;i++)
+		{
+			register long ii=i1+j1-di*i;
+			mreal a=A->vthr(ii),b=B->vthr(ii),c=C->vthr(ii);
+			mreal d=difr?-a*D->vthr(ii+di)+(2.-b)*D->vthr(ii)-c*D->vthr(ii-di):D->vthr(ii);
+			aa[i] = -c/(b+a*aa[i-1]);
+			bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
+		}
+		uu[j1+n*(n-1)-di*(j-1)] = bb[j];
+		for(long i=j-1;i>=0;i--)
+			uu[j1+n*(n-1)-di*i] = bb[i]+aa[i]*uu[j1+n*(n-1)-di*i-di];
+	}
+}
+//-----------------------------------------------------------------------------
+HMDT MGL_EXPORT mgl_data_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *how)
+{
+	const long nx=D->GetNx(),ny=D->GetNy(),nz=D->GetNz();
+	const long nn=nx*ny*nz, np=nx*ny, na=A->GetNN();
+	if(B->GetNN()!=na || C->GetNN()!=na)	return 0;
+	mglData *r = new mglData(nx,ny,nz);
+	bool per = mglchr(how,'c');
+	bool difr = mglchr(how,'d');
+	if(mglchr(how,'x') && (na==nn || na==np || na==nx))
+#pragma omp parallel
+	{
+		mglData T(nx,4);	mreal *uu=T.a+(per?3:2)*nx;
+#pragma omp for collapse(2)
+		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)
+		{
+			long i0=0;
+			if(na==nn)	i0=nx*(j+ny*k);	else if(na==np)	i0=nx*j;
+			if(per)	mgl_progonka_pr(A,B,C,D,T.a,nx,i0,1,difr);
+			else	mgl_progonka_sr(A,B,C,D,T.a,nx,i0,1,difr);
+			i0 = nx*(j+ny*k);
+			for(long i=0;i<nx;i++)	r->a[i+i0] = uu[i];
+		}
+	}
+	else if(mglchr(how,'y') && (na==nn || na==np || na==ny))
+#pragma omp parallel
+	{
+		mglData T(ny,4);	mreal *uu=T.a+(per?3:2)*ny;
+#pragma omp for collapse(2)
+		for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)
+		{
+			long i0=0;
+			if(na==nn)	i0=i+np*k;	else if(na==np)	i0=i;
+			if(per)	mgl_progonka_pr(A,B,C,D,T.a,ny,i0,nx,difr);
+			else	mgl_progonka_sr(A,B,C,D,T.a,ny,i0,nx,difr);
+			i0 = i+np*k;
+			for(long j=0;j<ny;j++)	r->a[j*nx+i0] = uu[j];
+		}
+	}
+	else if(mglchr(how,'z') && (na==nn || na==nz))
+#pragma omp parallel
+	{
+		mglData T(nz,4);	mreal *uu=T.a+(per?3:2)*nz;
+#pragma omp for collapse(2)
+		for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
+		{
+			long i0 = na==nn?i+nx*j:0;
+			if(per)	mgl_progonka_pr(A,B,C,D,T.a,nz,i0,np,difr);
+			else	mgl_progonka_sr(A,B,C,D,T.a,nz,i0,np,difr);
+			i0 = i+nx*j;
+			for(long k=0;k<nz;k++)	r->a[k*np+i0] = uu[k];
+		}
+	}
+	else if(mglchr(how,'h') && ny==nx && (na==nn || na==np) && nx>1)
+#pragma omp parallel
+	{
+		mglData T(np,2);
+#pragma omp for
+		for(long k=0;k<nz;k++)
+		{
+			mgl_progonka_hr(A,B,C,D,T.a,nx,na==nn ? k*np:0,difr);
+			memcpy(r->a+k*np, T.a+np, np*sizeof(mreal));
+		}
+	}
+	else	{	delete r;	r=0;	}
+	return r;
+}
+//-----------------------------------------------------------------------------
+uintptr_t MGL_EXPORT mgl_data_tridmat_(uintptr_t *A, uintptr_t *B, uintptr_t *C, uintptr_t *D, const char *how, int l)
+{	char *s=new char[l+1];	memcpy(s,how,l);	s[l]=0;
+	uintptr_t r = uintptr_t(mgl_data_tridmat(_DA_(A),_DA_(B),_DA_(C),_DA_(D),s));
+	delete []s;	return r;
+}
+//-----------------------------------------------------------------------------
+void MGL_NO_EXPORT mgl_progonka_sc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long i0, long di, bool difr)
+{
+	dual *aa=dat, *bb=dat+n, *uu=dat+2*n;
+	dual b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(i0);
+	if(difr)	d0 = (2.-b0)*d0-c0*D->vcthr(i0+di);
+	aa[0] = -c0/b0;	bb[0] = d0/b0;
+	for(long i=1;i<n;i++)
+	{
+		register long ii=i0+di*i;
+		dual a=A->vcthr(ii), b=B->vcthr(ii), c=C->vcthr(ii);
+		dual d=difr?-a*D->vcthr(ii-di)+(2.-b)*D->vcthr(ii)-c*D->vcthr(ii+di):D->vcthr(ii);
+		aa[i] = -c/(b+a*aa[i-1]);
+		bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
+	}
+	uu[n-1] = bb[n-1];
+	for(long i=n-2;i>=0;i--)	uu[i] = bb[i]+aa[i]*uu[i+1];
+}
+void MGL_NO_EXPORT mgl_progonka_pc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long i0, long di, bool difr)
+{
+	dual *aa=dat, *bb=dat+n, *gg=dat+2*n, *uu=dat+3*n;
+	dual a0=A->vcthr(i0), b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(i0);
+	if(difr)	d0 = -a0*D->vcthr(i0+di*(n-1))+(2.-b0)*d0-c0*D->vcthr(i0+di);
+	aa[0] =-c0/b0;	bb[0] = d0/b0;	gg[0] =-a0/b0;
+	for(long i=1;i<n;i++)
+	{
+		register long ii=i0+di*i, il=i0+di*((i+1)%n);
+		dual a=A->vcthr(ii), b=B->vcthr(ii), c=C->vcthr(ii);
+		dual d=difr?-a*D->vcthr(ii-di)+(2.-b)*D->vcthr(ii)-c*D->vcthr(il):D->vcthr(ii);
+		aa[i] = -c/(b+a*aa[i-1]);
+		bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
+		gg[i] = -a*gg[i-1]/(b+a*aa[i-1]);
+	}
+	dual P=bb[n-1]/(1.-gg[n-1]), Q=aa[n-1]/(1.-gg[n-1]);
+	aa[n-1] = Q;	bb[n-1] = P;
+	for(long i=n-2;i>=0;i--)
+	{
+		bb[i] += aa[i]*bb[i+1]+gg[i]*P;
+		aa[i] = aa[i]*aa[i+1]+gg[i]*Q;
+	}
+	dual u0 = bb[0]/(1.-aa[0]);
+	for(long i=0;i<n;i++)	uu[i]=bb[i]+aa[i]*u0;
+}
+void MGL_NO_EXPORT mgl_progonka_hc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long i0, bool difr)
+{
+	dual *aa=dat, *bb=dat+n, *uu=dat+n*n;
+	dual b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(i0);
+	uu[0] = d0/b0*(difr?(2.-b0):1.);
+	b0=B->vcthr(i0+n*n-1);	d0=D->vcthr(i0+n*n-1);
+	uu[n*n-1] = d0/b0*(difr?(2.-b0):1.);
+	long di = n-1, i1 = i0+n*(n-1);
+	// suppose the square grid!
+	for(long j=1;j<n;j++)
+	{
+		// first bottom-left triangle
+		b0=B->vcthr(i0+j);	c0=C->vcthr(i0+j);	d0=D->vcthr(i0+j);
+		if(difr)	d0 = (2.-b0)*d0-c0*D->vcthr(i0+j+di);
+		aa[0] = -c0/b0;	bb[0] = d0/b0;
+		for(long i=1;i<=j;i++)
+		{
+			register long ii=i0+j+di*i;
+			dual a=A->vcthr(ii),b=B->vcthr(ii),c=C->vcthr(ii);
+			dual d=difr?-a*D->vcthr(ii-di)+(2.-b)*D->vcthr(ii)-c*D->vcthr(ii+di):D->vcthr(ii);
+			aa[i] = -c/(b+a*aa[i-1]);
+			bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
+		}
+		uu[j+di*(j-1)] = bb[j];
+		for(long i=j-1;i>=0;i--)
+			uu[j+di*i] = bb[i]+aa[i]*uu[j+di*i+di];
+		// next top-right triangle
+		long j1=n-1-j;
+		b0=B->vcthr(i1+j1);	c0=C->vcthr(i1+j1);	d0=D->vcthr(i1+j1);
+		if(difr)	d0 = (2.-b0)*d0-c0*D->vcthr(i1+j1-di);
+		aa[0] = -c0/b0;	bb[0] = d0/b0;
+		for(long i=1;i<=j;i++)
+		{
+			register long ii=i1+j1-di*i;
+			dual a=A->vcthr(ii),b=B->vcthr(ii),c=C->vcthr(ii);
+			dual d=difr?-a*D->vcthr(ii+di)+(2.-b)*D->vcthr(ii)-c*D->vcthr(ii-di):D->vcthr(ii);
+			aa[i] = -c/(b+a*aa[i-1]);
+			bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
+		}
+		uu[j1+n*(n-1)-di*(j-1)] = bb[j];
+		for(long i=j-1;i>=0;i--)
+			uu[j1+n*(n-1)-di*i] = bb[i]+aa[i]*uu[j1+n*(n-1)-di*i-di];
+	}
+}
+//-----------------------------------------------------------------------------
+HADT MGL_EXPORT mgl_datac_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *how)
+{
+	const long nx=D->GetNx(),ny=D->GetNy(),nz=D->GetNz();
+	const long nn=nx*ny*nz, np=nx*ny, na=A->GetNN();
+	if(B->GetNN()!=na || C->GetNN()!=na)	return 0;
+	mglDataC *r = new mglDataC(nx,ny,nz);
+	bool per = mglchr(how,'c');
+	bool difr = mglchr(how,'d');
+	if(mglchr(how,'x') && (na==nn || na==np || na==nx))
+#pragma omp parallel
+	{
+		mglDataC T(nx,4);	dual *uu=T.a+(per?3:2)*nx;
+#pragma omp for collapse(2)
+		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)
+		{
+			long i0=0;
+			if(na==nn)	i0=nx*(j+ny*k);	else if(na==np)	i0=nx*j;
+			if(per)	mgl_progonka_pc(A,B,C,D,T.a,nx,i0,1,difr);
+			else	mgl_progonka_sc(A,B,C,D,T.a,nx,i0,1,difr);
+			i0 = nx*(j+ny*k);
+			for(long i=0;i<nx;i++)	r->a[i+i0] = uu[i];
+		}
+	}
+	else if(mglchr(how,'y') && (na==nn || na==np || na==ny))
+#pragma omp parallel
+	{
+		mglDataC T(ny,4);	dual *uu=T.a+(per?3:2)*ny;
+#pragma omp for collapse(2)
+		for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)
+		{
+			long i0=0;
+			if(na==nn)	i0=i+np*k;	else if(na==np)	i0=i;
+			if(per)	mgl_progonka_pc(A,B,C,D,T.a,ny,i0,nx,difr);
+			else	mgl_progonka_sc(A,B,C,D,T.a,ny,i0,nx,difr);
+			i0 = i+np*k;
+			for(long j=0;j<ny;j++)	r->a[j*nx+i0] = uu[j];
+		}
+	}
+	else if(mglchr(how,'z') && (na==nn || na==nz))
+#pragma omp parallel
+	{
+		mglDataC T(nz,4);	dual *uu=T.a+(per?3:2)*nz;
+#pragma omp for collapse(2)
+		for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
+		{
+			long i0 = na==nn?i+nx*j:0;
+			if(per)	mgl_progonka_pc(A,B,C,D,T.a,nz,i0,np,difr);
+			else	mgl_progonka_sc(A,B,C,D,T.a,nz,i0,np,difr);
+			i0 = i+nx*j;
+			for(long k=0;k<nz;k++)	r->a[k*np+i0] = uu[k];
+		}
+	}
+	else if(mglchr(how,'h') && ny==nx && (na==nn || na==np) && nx>1)
+#pragma omp parallel
+	{
+		mglDataC T(np,2);
+#pragma omp for
+		for(long k=0;k<nz;k++)
+		{
+			mgl_progonka_hc(A,B,C,D,T.a,nx,na==nn ? k*np:0,difr);
+			memcpy(r->a+k*np, T.a+np, np*sizeof(dual));
+		}
+	}
+	else	{	delete r;	r=0;	}
+	return r;
+}
+//-----------------------------------------------------------------------------
+uintptr_t MGL_EXPORT mgl_datac_tridmat_(uintptr_t *A, uintptr_t *B, uintptr_t *C, uintptr_t *D, const char *how, int l)
+{	char *s=new char[l+1];	memcpy(s,how,l);	s[l]=0;
+	uintptr_t r = uintptr_t(mgl_datac_tridmat(_DA_(A),_DA_(B),_DA_(C),_DA_(D),s));
+	delete []s;	return r;
+}
+//-----------------------------------------------------------------------------
