@@ -1,6 +1,6 @@
 /***************************************************************************
  * surf.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -22,7 +22,6 @@
 #include "mgl2/data.h"
 #include "mgl2/eval.h"
 #include "mgl2/base.h"
-//#include <vector>
 //-----------------------------------------------------------------------------
 //
 //	CloudQ series
@@ -108,6 +107,21 @@ void MGL_EXPORT mgl_cloud_(uintptr_t *gr, uintptr_t *a, const char *sch, const c
 //	Surf3 series
 //
 //-----------------------------------------------------------------------------
+mreal MGL_NO_EXPORT mgl_get_norm(mreal x, mreal d1, mreal d2, mreal d3)
+{
+	mreal nx = d1*(1-x) + d2*x;
+	if(mgl_isbad(nx))
+	{
+		nx = d1*(1+x) - d3*x;
+		if(mgl_isbad(nx))
+		{
+			if(mgl_isfin(d1))	nx = d1;
+			if(mgl_isfin(d2))	nx = d2;
+			if(mgl_isfin(d3))	nx = d3;
+		}
+	}
+	return nx;
+}
 mglPoint MGL_NO_EXPORT mgl_normal_3d(HCDT a, mglPoint p, bool inv, long n,long m,long l)
 {
 	register long i,j,k;
@@ -117,18 +131,17 @@ mglPoint MGL_NO_EXPORT mgl_normal_3d(HCDT a, mglPoint p, bool inv, long n,long m
 	i = i<n-1 ? i:n-2;	j = j<m-1 ? j:m-2;	k = k<l-1 ? k:l-2;
 	x-=i;	y-=j;	z-=k;
 
-	nx = a->dvx(i,j,k)*(1-x) + a->dvx(i+1,j,k)*x;
-	ny = a->dvy(i,j,k)*(1-y) + a->dvy(i,j+1,k)*y;
-	nz = a->dvz(i,j,k)*(1-z) + a->dvz(i,j,k+1)*z;
+	nx = mgl_get_norm(x, a->dvx(i,j,k), a->dvx(i+1,j,k), i>0?a->dvx(i-1,j,k):NAN);
+	ny = mgl_get_norm(y, a->dvy(i,j,k), a->dvy(i,j+1,k), j>0?a->dvy(i,j-1,k):NAN);
+	nz = mgl_get_norm(z, a->dvz(i,j,k), a->dvz(i,j,k+1), k>0?a->dvz(i,j,k-1):NAN);
 	return inv ? mglPoint(nx,ny,nz) : mglPoint(-nx,-ny,-nz);
 }
 //-----------------------------------------------------------------------------
-mreal MGL_NO_EXPORT mgl_normal_1d(HCDT a, mreal x, bool inv, long n)
+mreal MGL_NO_EXPORT mgl_normal_1d(HCDT a, mreal x, long n)
 {
-	register long i=long(x);	x-=i;
-	mreal nx = a->dvx(i);
-	if(i<n-1)	nx = nx*(1-x) + a->dvx(i+1)*x;
-	return inv ? nx : -nx;
+	register long i=long(x);
+	i = i<n-1 ? i:n-2;	x-=i;
+	return mgl_get_norm(x, a->dvx(i), a->dvx(i+1), i>0?a->dvx(i-1):NAN);
 }
 //-----------------------------------------------------------------------------
 mglPoint MGL_NO_EXPORT mgl_find_norm(bool nboth, HCDT x, HCDT y, HCDT z, HCDT a, mglPoint u, bool inv, long n,long m,long l)
@@ -136,9 +149,9 @@ mglPoint MGL_NO_EXPORT mgl_find_norm(bool nboth, HCDT x, HCDT y, HCDT z, HCDT a,
 	mglPoint s = mgl_normal_3d(a,u,inv,n,m,l), t, q;
 	if(nboth)
 	{
-		q.x = s.x/mgl_normal_1d(x,u.x,true,n);
-		q.y = s.y/mgl_normal_1d(y,u.y,true,m);
-		q.z = s.z/mgl_normal_1d(z,u.z,true,l);
+		q.x = s.x/mgl_normal_1d(x,u.x,n);
+		q.y = s.y/mgl_normal_1d(y,u.y,m);
+		q.z = s.z/mgl_normal_1d(z,u.z,l);
 	}
 	else
 	{
@@ -262,6 +275,7 @@ void MGL_NO_EXPORT mgl_surf3ca_gen(HMGL gr, double val, HCDT x, HCDT y, HCDT z,
 		{
 			register long i1 = i+n*j;
 			mreal a0 = a->v(i,j,k);
+			if(mgl_isnan(a0))	continue;
 			if(i<n-1)
 			{
 				mreal d = mgl_d(val,a0,a->v(i+1,j,k));
@@ -275,7 +289,7 @@ void MGL_NO_EXPORT mgl_surf3ca_gen(HMGL gr, double val, HCDT x, HCDT y, HCDT z,
 				{	ky2[i1] = kk.size();	kk.push_back(mglPoint(i,j+d,k));	}
 			}
 			if(k>0)
-			{	
+			{
 				mreal d = mgl_d(val,a->v(i,j,k-1),a0);
 				if(d>=0 && d<1)
 				{	kz[i1] = kk.size();		kk.push_back(mglPoint(i,j,k+d-1));	}
@@ -285,36 +299,38 @@ void MGL_NO_EXPORT mgl_surf3ca_gen(HMGL gr, double val, HCDT x, HCDT y, HCDT z,
 		if(b && c)	for(size_t i=kk1;i<kk.size();i++)
 		{
 			mglPoint &u = kk[i];
-			u.c = gr->AddPnt(nboth ? mglPoint(x->linear(u.x,0,0),y->linear(u.y,0,0),z->linear(u.z,0,0)) : 
+			mreal cc = c->linear(u.x,u.y,u.z), bb = b->linear(u.x,u.y,u.z);
+			if(mgl_isnan(cc) || mgl_isnan(bb))	u.c = -1;	else
+			u.c = gr->AddPnt(nboth ? mglPoint(x->linear(u.x,0,0),y->linear(u.y,0,0),z->linear(u.z,0,0)) :
 					mglPoint(x->linear(u.x,u.y,u.z),y->linear(u.x,u.y,u.z),z->linear(u.x,u.y,u.z)),
-					gr->GetC(ss,c->linear(u.x,u.y,u.z)),
-					mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l),
-					gr->GetA(b->linear(u.x,u.y,u.z)));
+					gr->GetC(ss,cc), mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l), gr->GetA(bb));
 		}
 		else if(c)	for(size_t i=kk1;i<kk.size();i++)
 		{
 			mglPoint &u = kk[i];
-			u.c = gr->AddPnt(nboth ? mglPoint(x->linear(u.x,0,0),y->linear(u.y,0,0),z->linear(u.z,0,0)) : 
+			mreal cc = c->linear(u.x,u.y,u.z);
+			if(mgl_isnan(cc))	u.c = -1;	else
+			u.c = gr->AddPnt(nboth ? mglPoint(x->linear(u.x,0,0),y->linear(u.y,0,0),z->linear(u.z,0,0)) :
 					mglPoint(x->linear(u.x,u.y,u.z),y->linear(u.x,u.y,u.z),z->linear(u.x,u.y,u.z)),
-					gr->GetC(ss,c->linear(u.x,u.y,u.z)),
-					mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l));
+					gr->GetC(ss,cc), mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l));
 		}
 		else if(b)	for(size_t i=kk1;i<kk.size();i++)
 		{
 			mglPoint &u = kk[i];
-			u.c = gr->AddPnt(nboth ? mglPoint(x->linear(u.x,0,0),y->linear(u.y,0,0),z->linear(u.z,0,0)) : 
+			mreal bb = b->linear(u.x,u.y,u.z);
+			if(mgl_isnan(bb))	u.c = -1;	else
+			u.c = gr->AddPnt(nboth ? mglPoint(x->linear(u.x,0,0),y->linear(u.y,0,0),z->linear(u.z,0,0)) :
 					mglPoint(x->linear(u.x,u.y,u.z),y->linear(u.x,u.y,u.z),z->linear(u.x,u.y,u.z)),
-					cv, mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l),
-					gr->GetA(b->linear(u.x,u.y,u.z)));
+					cv, mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l), gr->GetA(bb));
 		}
 		else	for(size_t i=kk1;i<kk.size();i++)
 		{
 			mglPoint &u = kk[i];
-			u.c = gr->AddPnt(nboth ? mglPoint(x->linear(u.x,0,0),y->linear(u.y,0,0),z->linear(u.z,0,0)) : 
+			u.c = gr->AddPnt(nboth ? mglPoint(x->linear(u.x,0,0),y->linear(u.y,0,0),z->linear(u.z,0,0)) :
 					mglPoint(x->linear(u.x,u.y,u.z),y->linear(u.x,u.y,u.z),z->linear(u.x,u.y,u.z)),
 					cv, mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l));
 		}
-		
+
 		if(k>0)	mgl_surf3_plot(gr,n,m,kx1,kx2,ky1,ky2,kz,kk,wire);
 	}
 	delete []kx1;	delete []kx2;	delete []ky1;
@@ -408,20 +424,21 @@ void MGL_EXPORT mgl_surf3a_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT a, HCDT b,
 	long num = mgl_isnan(r)?3:long(r+0.5);
 	if(b->GetNx()==num && b->GetNy()==1 && b->GetNz()==1)
 	{
-		mreal v,a0=gr->AlphaDef;
+		mreal a0=gr->AlphaDef;
 		for(long i=0;i<num;i++)
 		{
-			v = gr->Max.c + (gr->Min.c-gr->Max.c)*(i+1.)/(num+1);
+			mreal v = gr->Max.c + (gr->Min.c-gr->Max.c)*(i+1.)/(num+1);
 			gr->AlphaDef = gr->GetA(b->v(i));
 			mgl_surf3_xyz_val(gr,v,x,y,z,a,sch,0);
 		}
 		gr->AlphaDef = a0;
 	}
-	else for(long i=0;i<num;i++)
-	{
-		mreal v = gr->Max.c + (gr->Min.c-gr->Max.c)*(i+1.)/(num+1);
-		mgl_surf3a_xyz_val(gr,v,x,y,z,a,b,sch,0);
-	}
+	else
+		for(long i=0;i<num;i++)
+		{
+			mreal v = gr->Max.c + (gr->Min.c-gr->Max.c)*(i+1.)/(num+1);
+			mgl_surf3a_xyz_val(gr,v,x,y,z,a,b,sch,0);
+		}
 	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
@@ -604,50 +621,43 @@ void MGL_EXPORT mgl_beam_val(HMGL gr, double val, HCDT tr, HCDT g1, HCDT g2, HCD
 	if(tr->GetNx()<3 || tr->GetNy()<n || g1->GetNx()<3 || g1->GetNy()<n || g2->GetNx()<3 || g2->GetNy()<n)
 	{	gr->SetWarn(mglWarnDim,"Beam");	return;	}
 	mglData x(a),y(a),z(a),b(a);
-	register long i,j,k,i0;
 	mreal asum0=1;	r = fabs(r);
-	if(flag & 4)	for(j=0;j<m*l;j++)	asum0 += a->vthr(j)*a->vthr(j);
+	if(flag & 4)	for(long j=0;j<m*l;j++)	asum0 += a->vthr(j)*a->vthr(j);
 	if(asum0==0)	{	gr->SetWarn(mglWarnZero,"Beam");	return;	}
-	for(i=0;i<n;i++)
+#pragma omp parallel for
+	for(long i=0;i<n;i++)
 	{
-		if(gr->NeedStop())	break;
+		if(gr->NeedStop())	continue;
 		if(flag & 4)
 		{
 			mreal asum=0, amax=0;
-			for(j=0;j<m*l;j++)
+			for(long j=0;j<m*l;j++)
 			{
 				register mreal aa = a->vthr(j+m*l*i);
 				asum += aa*aa;	amax = amax>aa ? amax : aa;
 			}
 			amax = amax?sqrt(asum/asum0)/amax:0;
-#pragma omp parallel for
-			for(j=0;j<m*l;j++)	b.a[j+m*l*i] = b.a[j+m*l*i]*amax;
+			for(long j=0;j<m*l;j++)	b.a[j+m*l*i] = b.a[j+m*l*i]*amax;
+		}
+		if(flag & 1)	for(long j=0;j<m;j++)	for(long k=0;k<l;k++)
+		{
+			register long i0 = j+m*(k+l*i);
+			x.a[i0] = 2*j/(m-1.)-1;
+			y.a[i0] = 2*k/(l-1.)-1;
+			z.a[i0] = gr->Max.z*i/(n-1.);
+		}
+		else	for(long j=0;j<m;j++)	for(long k=0;k<l;k++)
+		{
+			register long i0 = j+m*(k+l*i);
+			x.a[i0] = tr->v(0,i) + g1->v(0,i)*(2*j/(m-1.)-1)*r + g2->v(0,i)*(2*k/(l-1.)-1)*r;
+			y.a[i0] = tr->v(1,i) + g1->v(1,i)*(2*j/(m-1.)-1)*r + g2->v(1,i)*(2*k/(l-1.)-1)*r;
+			z.a[i0] = tr->v(2,i) + g1->v(2,i)*(2*j/(m-1.)-1)*r + g2->v(2,i)*(2*k/(l-1.)-1)*r;
+		}
+		if(flag & 2)	for(long j=0;j<m;j++)	for(long k=0;k<l;k++)
+		{
+			register long i0 = j+m*(k+l*i);
+			x.a[i0] = hypot(x.a[i0],y.a[i0]);
 		}
-		if(flag & 1)
-#pragma omp parallel for collapse(2)
-			for(j=0;j<m;j++)	for(k=0;k<l;k++)
-			{
-				i0 = j+m*(k+l*i);
-				x.a[i0] = 2*j/(m-1.)-1;
-				y.a[i0] = 2*k/(l-1.)-1;
-				z.a[i0] = gr->Max.z*i/(n-1.);
-			}
-		else
-#pragma omp parallel for collapse(2)
-			for(j=0;j<m;j++)	for(k=0;k<l;k++)
-			{
-				i0 = j+m*(k+l*i);
-				x.a[i0] = tr->v(0,i) + g1->v(0,i)*(2*j/(m-1.)-1)*r + g2->v(0,i)*(2*k/(l-1.)-1)*r;
-				y.a[i0] = tr->v(1,i) + g1->v(1,i)*(2*j/(m-1.)-1)*r + g2->v(1,i)*(2*k/(l-1.)-1)*r;
-				z.a[i0] = tr->v(2,i) + g1->v(2,i)*(2*j/(m-1.)-1)*r + g2->v(2,i)*(2*k/(l-1.)-1)*r;
-			}
-		if(flag & 2)
-#pragma omp parallel for collapse(2)
-			for(j=0;j<m;j++)	for(k=0;k<l;k++)
-			{
-				long i0 = j+m*(k+l*i);
-				x.a[i0] = hypot(x.a[i0],y.a[i0]);
-			}
 	}
 	mgl_surf3_xyz_val(gr,val,&x,&y,&z,&b,stl,0);
 }
