@@ -1,6 +1,6 @@
 /***************************************************************************
  * qt.cpp is part of Math Graphic Library                              *
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -330,8 +330,15 @@ void QMathGL::update()
 		else if(draw)	{	mglGraph g(gr);	draw->Draw(&g);	}
 		if(mgl_is_frames(gr))	mgl_end_frame(gr);
 		setlocale(LC_NUMERIC, "");
-		gr->AskStop(false);	afterPlot();
+		gr->AskStop(false);
 	}
+	else if(mgl_get_num_frame(gr)>0)
+	{
+		mgl_set_alpha(gr,alpha);	mgl_set_light(gr,light);
+//		mgl_zoom(gr,x1,y1,x2,y2);	mgl_view(gr,-phi,-tet,0);
+		mgl_get_frame(gr,0);
+	}
+	afterPlot();
 }
 //-----------------------------------------------------------------------------
 void QMathGL::afterPlot()
@@ -1324,6 +1331,11 @@ HMGL MGL_EXPORT mgl_create_graph_qt(int (*draw)(HMGL gr, void *p), const char *t
 	g->Window(0,0,draw,title,par,load);
 	return g;
 }
+void* mgl_qt_widget(HMGL gr)
+{
+	mglCanvasQT *g = dynamic_cast<mglCanvasQT *>(gr);
+	return g?g->QMGL:NULL;
+}
 int MGL_EXPORT mgl_qt_run()	{	return (qApp)?qApp->exec():-1;	}
 //-----------------------------------------------------------------------------
 uintptr_t MGL_EXPORT mgl_create_graph_qt_(const char *title, int l)
